#ifndef REACTORDETECTOR_HH
#define REACTORDETECTOR_HH
#include <math.h>
#include <iostream.h>
#include <string.h>
#include "ReactorEvent.hh"
#include "ReactorFortran.hh"
#include "ReactorConstants.hh"

class ReactorDetector{
  //
  // Simple detector simulation implementation
  //
public:
  //
  // Basic constructors and operators
  //
  ReactorDetector();
  ~ReactorDetector();
  ReactorDetector(const ReactorDetector& Detector);
  ReactorDetector& operator=(const ReactorDetector& rhs);
  //
  // Member function to simulate events.
  //
  void LightsOut(ReactorEvent& Event);
  //
  // Access functions
  //
  double GetPHpositron() const {return PHpositron;}
  double GetPHneutron() const {return PHneutron;}
  double GetPHposReco() const {return PHposReco;}
  double GetPHneuReco() const {return PHneuReco;}

  double GetMeanNpe()const {return MeanNpe;}
  double GetMeanAtten() const {return MeanAtten;}
  double GetMeanSfactor() const {return MeanSfactor;}

  int GetNgamma() const {return Ngamma;}
  double* const GetEgamma() {return Egamma;} 
  double* const GetTgamma() {return Tgamma;} 
  int GetNgamma(char parent);
  void GetGamma(char parent,double* xyz);
  void GetGamma(char parent,double* e,double* t,double* r,
		double* c, double* f);

  int GetNpmt() const {return Npmt;}
  void ReactorDetector::GetPMTdata(int npmt,double* XYZpmt,
				   int& npos,double* PHpos,double* Tpos,
				   int& nneu,double* PHneu,double* Tneu);


private:

  static ReactorConstants RC;

  int Npmt;
  int Ncosbins;
  int Nphibins;

  double* Xpmt;
  double* Ypmt;
  double* Zpmt;

  double PHpositron;
  double PHneutron;

  double PHposReco;
  double PHneuReco;

  double TimePositron;
  double TimeNeutron;

  double MeanNpe;
  double MeanAtten;
  double MeanSfactor;

  int Ngamma;
  double* Rgamma;
  double* Egamma;
  double* Tgamma;
  char* Ogamma;

  class PMT;
  int PMTsize;
  int PMTlen;
  PMT* PMTdata;
  PMT* ResizePMT();

  void GenerateSecondaries(ReactorEvent& Event);
  void GenerateResponse(ReactorEvent& Event);
  void GenerateGammas(ReactorEvent& Event);

  class PMT{
  public:
    PMT(){;};
    ~PMT(){;};
    PMT(const PMT& p);
    PMT& operator=(const PMT& rhs);

    int npmt;
    double xyz[3]; 
    double TimeGen;
    double TimeSmr;
    double PHgen;
    double PHsmr;
    double PathGd;
    double PathSc;
    double Atten;
    double Sfactor;
    int Npe;
    double Xpe;
    char Parent;
  };
};
//
// constructor
//
ReactorDetector::ReactorDetector(){
  //
  // determine the number of PMT
  //
  Npmt = 16*RC.R2*RC.R2*RC.PMTcoverage/RC.PMTdiameter/RC.PMTdiameter;
  Ncosbins = sqrt(Npmt/RC.pi);
  Nphibins = sqrt(Npmt*RC.pi);
  Npmt = Ncosbins*Nphibins;

  Xpmt = new double[Npmt];
  Ypmt = new double[Npmt];
  Zpmt = new double[Npmt];

  //
  // Make a guess at PMT size to avoid too much re-sizing
  //
  PMTlen = 0;
  PMTsize = 10*Npmt;
  PMTdata = new PMT[PMTsize];
  //
  // loop over cotheta, then phi
  //
  double* xpmt=Xpmt;
  double* ypmt=Ypmt;
  double* zpmt=Zpmt;

  double dphi = 2*RC.pi/Nphibins;
  double dz = 2.0/Ncosbins;

  for(int i=0;i<Ncosbins;i++){
    double z = -1+i*dz+dz/2;
    for(int j=0;j<Nphibins;j++){
      double phi = j*dphi+dphi/2;
      *xpmt++ = RC.R2*sqrt(1-z*z)*cos(phi);
      *ypmt++ = RC.R2*sqrt(1-z*z)*sin(phi);
      *zpmt++ = RC.R2*z;
      cout << "z,phi  " << z << " " << phi << endl;
    }
  }	
  //
  // initialize gamma arrays
  //
  Ngamma = 0;
  Rgamma = new double[Ngamma];
  Egamma = new double[Ngamma];
  Tgamma = new double[Ngamma];
  Ogamma = new char[Ngamma];

}
//
// destructor
//
ReactorDetector::~ReactorDetector(){
  delete[] Xpmt;
  delete[] Ypmt;
  delete[] Zpmt;
  delete[] PMTdata;
  delete[] Rgamma;
  delete[] Egamma;
  delete[] Tgamma;
  delete[] Ogamma;
}
//
// copy constructor
//
ReactorDetector::ReactorDetector(const ReactorDetector& Detector){

  Npmt = Detector.Npmt;
  Ncosbins = Detector.Ncosbins;
  Nphibins = Detector.Nphibins;

  Xpmt = new double[Npmt];
  Ypmt = new double[Npmt];  
  Zpmt = new double[Npmt];

  double* pold = Detector.Xpmt+Npmt;
  double* pnew = Xpmt+Npmt;
  while(pnew>Ypmt)*--pnew=*--pold;

  pold = Detector.Ypmt+Npmt;
  pnew = Ypmt+Npmt;
  while(pnew>Ypmt)*--pnew=*--pold;

  pold = Detector.Zpmt+Npmt;
  pnew = Zpmt+Npmt;
  while(pnew>Zpmt)*--pnew=*--pold;

  PHpositron = Detector.PHpositron;
  PHneutron = Detector.PHneutron;

  PHposReco = Detector.PHposReco;
  PHneuReco = Detector.PHneuReco;

  TimePositron = Detector.TimePositron;
  TimeNeutron = Detector.TimeNeutron;

  MeanNpe = Detector.MeanNpe;
  MeanAtten = Detector.MeanAtten;
  MeanSfactor = Detector.MeanSfactor;

  Ngamma = Detector.Ngamma;

  Rgamma = new double[Ngamma];
  pold = Detector.Rgamma+3*Ngamma;
  pnew = Rgamma+3*Ngamma;
  while(pnew>Rgamma)*--pnew=*--pold;

  Egamma = new double[Ngamma];
  pold = Detector.Egamma+Ngamma;
  pnew = Egamma+Ngamma;
  while(pnew>Egamma)*--pnew=*--pold;

  Tgamma = new double[Ngamma];
  pold = Detector.Tgamma+Ngamma;
  pnew = Tgamma+Ngamma;
  while(pnew>Tgamma)*--pnew=*--pold;

  Ogamma = new char[Ngamma];
  char* rold = Detector.Ogamma+Ngamma;
  char* rnew = Ogamma+Ngamma;
  while(rnew>Ogamma)*--rnew=*--rold;

  PMTsize = Detector.PMTsize;
  PMTlen = Detector.PMTlen;
  PMTdata = new PMT[PMTsize];
  PMT* qold = Detector.PMTdata+PMTsize;
  PMT* qnew = PMTdata+PMTsize;
  while(qnew>PMTdata)*--qnew=*--qold;
}    
//
// = operator
//
ReactorDetector& ReactorDetector::operator=(const ReactorDetector& rhs){

  if(this == &rhs)return *this;

  Npmt = rhs.Npmt;
  Ncosbins = rhs.Ncosbins;
  Nphibins = rhs.Nphibins;

  delete[] Xpmt;
  delete[] Ypmt;
  delete[] Zpmt;

  Xpmt = new double[Npmt];
  Ypmt = new double[Npmt];  
  Zpmt = new double[Npmt];

  double* pold = rhs.Xpmt+Npmt;
  double* pnew = Xpmt+Npmt;
  while(pnew>Ypmt)*--pnew=*--pold;

  pold = rhs.Ypmt+Npmt;
  pnew = Ypmt+Npmt;
  while(pnew>Ypmt)*--pnew=*--pold;

  pold = rhs.Zpmt+Npmt;
  pnew = Zpmt+Npmt;
  while(pnew>Zpmt)*--pnew=*--pold;

  PHpositron = rhs.PHpositron;
  PHneutron = rhs.PHneutron;

  PHposReco = rhs.PHposReco;
  PHneuReco = rhs.PHneuReco;

  TimePositron = rhs.TimePositron;
  TimeNeutron = rhs.TimeNeutron;

  MeanNpe = rhs.MeanNpe;
  MeanAtten = rhs.MeanAtten;
  MeanSfactor = rhs.MeanSfactor;

  Ngamma = rhs.Ngamma;

  delete[] Rgamma;
  Rgamma = new double[Ngamma];
  pold = rhs.Rgamma+3*Ngamma;
  pnew = Rgamma+3*Ngamma;
  while(pnew>Rgamma)*--pnew=*--pold;

  delete[] Egamma;
  Egamma = new double[Ngamma];
  pold = rhs.Egamma+Ngamma;
  pnew = Egamma+Ngamma;
  while(pnew>Egamma)*--pnew=*--pold;

  delete[] Tgamma;
  Tgamma = new double[Ngamma];
  pold = rhs.Tgamma+Ngamma;
  pnew = Tgamma+Ngamma;
  while(pnew>Tgamma)*--pnew=*--pold;

  delete[] Ogamma;
  Ogamma = new char[Ngamma];
  char* rold = rhs.Ogamma+Ngamma;
  char* rnew = Ogamma+Ngamma;
  while(rnew>Ogamma)*--rnew=*--rold;

  PMTsize = rhs.PMTsize;
  PMTlen = rhs.PMTlen;
  delete[] PMTdata;
  PMTdata = new PMT[PMTsize];
  PMT* qold = rhs.PMTdata+PMTsize;
  PMT* qnew = PMTdata+PMTsize;
  while(qnew>PMTdata)*--qnew=*--qold;

  return *this;

}
//
// PMT sub-class copy constructor
//    
ReactorDetector::PMT::PMT(const ReactorDetector::PMT& p){
  
  npmt=p.npmt; 
  for (int i=0;i<3;i++)xyz[i]=p.xyz[i]; 
  TimeGen=p.TimeGen;
  TimeSmr=p.TimeSmr;
  PHgen=p.PHgen;
  PHsmr=p.PHsmr;
  PathGd=p.PathGd;
  PathSc=p.PathSc;
  Sfactor=p.Sfactor;
  Atten=p.Atten;
  Npe=p.Npe;
  Xpe=p.Xpe;
  Parent=p.Parent;
}
//
// PMT sub-class equal operator
//    
ReactorDetector::PMT& 
ReactorDetector::PMT::operator=(const ReactorDetector::PMT& rhs){

  if(this == &rhs)return *this;

  npmt=rhs.npmt;
  for (int i=0;i<3;i++)xyz[i]=rhs.xyz[i]; 
  TimeGen=rhs.TimeGen;
  TimeSmr=rhs.TimeSmr;
  PHgen=rhs.PHgen;
  PHsmr=rhs.PHsmr;
  PathGd=rhs.PathGd;
  PathSc=rhs.PathSc;
  Sfactor=rhs.Sfactor;
  Atten=rhs.Atten;
  Npe=rhs.Npe;
  Xpe=rhs.Xpe;
  Parent=rhs.Parent;

  return *this;
}
//
// Detector simulation function
//
void ReactorDetector::LightsOut(ReactorEvent& Event){

  PMTlen=0;
  delete[] PMTdata;
  PMTdata = new PMT[PMTsize];
  
  GenerateSecondaries(Event);
  GenerateGammas(Event);
  GenerateResponse(Event);
}
//
// Access to PMT data
//
void ReactorDetector::GetPMTdata(int npmt,double* XYZpmt,
				 int& npos,double* PHpos,double* Tpos,
				 int& nneu,double* PHneu,double* Tneu){
  
  *(XYZpmt+0) = *(Xpmt+npmt);
  *(XYZpmt+1) = *(Ypmt+npmt);
  *(XYZpmt+2) = *(Zpmt+npmt);

  npos=0;
  nneu=0;
  for(PMT* PMTptr=PMTdata;PMTptr<PMTdata+PMTlen;PMTptr++){
    if(npmt==(*PMTptr).npmt){
      if((*PMTptr).Parent=='P'){
	*(PHpos+npos) = (*PMTptr).PHsmr;
	*(Tpos+npos) = (*PMTptr).TimeSmr;
	npos++;
      }
      if((*PMTptr).Parent=='N'){
	*(PHneu+nneu) = (*PMTptr).PHsmr;
	*(Tneu+nneu) = (*PMTptr).TimeSmr;
	nneu++;
      }
    } 
  }
  return;
}
void ReactorDetector::GenerateSecondaries(ReactorEvent& Event){

  //
  // Displace the positron vertex by its range along the positron direction.
  //
  double kepos = Event.GetPositronKE();
  double rangepos;
  double tpos;
  PositronRange(kepos,rangepos,tpos);
  //
  double cospos = Event.GetPositronCosTheta();
  double phipos = Event.GetPositronPhi();
  double rint[3];
  for(int i=0;i<3;i++)rint[i] = *(Event.GetNubarVertexXYZ()+i);
  //
  double rpos[3];
  rpos[0]=rint[0]+rangepos*sqrt(1-cospos*cospos)*cos(phipos);
  rpos[1]=rint[1]+rangepos*sqrt(1-cospos*cospos)*sin(phipos);
  rpos[2]=rint[2]+rangepos*cospos;
  double rrpos = sqrt(rpos[0]*rpos[0]+rpos[1]*rpos[1]+rpos[2]*rpos[2]);
  //
  // Throw the neutron vertex via a simple Gaussian
  //
  float rn[3];
  int len=3;
  rnormx_(rn,len,ranlux_);
  //
  //  Simple scheme that is not right at boundaries
  //  Neglects discontinuity in mean free path;
  //  Time will be taken as protportional to sqrt(path)
  //
  double rneu[3];
  double rrint = sqrt(rint[0]*rint[0]+rint[1]*rint[1]+rint[2]*rint[2]);
  double path;
  double tneu;
  if(rrint<=RC.R0){
    for (int i=0;i<3;i++)rneu[i]=rint[i]+RC.mfpGd*rn[i]/sqrt(3.);
    path = RC.mfpGd*sqrt(rn[0]*rn[0]+rn[1]*rn[1]+rn[2]*rn[2]);
    tneu = RC.tGd*sqrt(path/RC.mfpGd);
  }
  else if(rrint<RC.R2){
    for (int i=0;i<3;i++)rneu[i]=rint[i]+RC.mfpSc*rn[i]/sqrt(3.);
    path = RC.mfpSc*sqrt(rn[0]*rn[0]+rn[1]*rn[1]+rn[2]*rn[2]);
    tneu = RC.tSc*sqrt(path/RC.mfpSc);
  }
  else{
    for (int i=0;i<3;i++)rneu[i]=999999;
    tneu = 999999.;
  }
  double rrneu = sqrt(rneu[0]*rneu[0]+rneu[1]*rneu[1]+rneu[2]*rneu[2]);
  //
  //  subevent times
  //
  double T[2] = {tpos,tneu};
  TimePositron = T[0];
  TimeNeutron = T[1];
  //
  // set positron,neutron vertex in Event class
  //
  double Rpos[7];
  Rpos[0] = tpos;
  for(int i=1;i<4;i++)Rpos[i]=rpos[i-1];
  Rpos[4] = rrpos;
  Rpos[5] = rpos[2]/rrpos;
  Rpos[6] = atan2(rpos[1],rpos[0]);
  if(Rpos[6]<0)Rpos[6]+=2*RC.pi;
  Event.SetPositronVertex(Rpos);
  //
  double Rneu[7];
  Rneu[0] = tneu;
  for(int i=1;i<4;i++)Rneu[i]=rneu[i-1];
  Rneu[4] = rrneu;
  Rneu[5] = rneu[2]/rrneu;
  Rneu[6] = atan2(rneu[1],rneu[0]);
  if(Rneu[6]<0)Rneu[6]+=2*RC.pi;
  Event.SetNeutronVertex(Rneu);
}

void ReactorDetector::GenerateResponse(ReactorEvent& Event){

  PHpositron = 0;
  PHneutron = 0;
  PHposReco = 0;
  PHneuReco = 0;
  MeanSfactor = 0;
  MeanAtten = 0;
  MeanNpe = 0;
  //
  //
  //
  int pmtindex=0;
  double* xpmt=Xpmt;
  double* ypmt=Ypmt;
  double* zpmt=Zpmt;
  
  PMT* PMTptr = PMTdata;

  for(int i=0;i<Ncosbins;i++){
    for(int j=0;j<Nphibins;j++){
      double rpmt[3];
      rpmt[0] = *xpmt++;
      rpmt[1] = *ypmt++;
      rpmt[2] = *zpmt++;
      double r[3];
      double dr[3];
      double tr[3];
      
      double sumpos[4]={0,0,0,0};
      double sumneu[4]={0,0,0,0};

      double* rgptr=Rgamma;
      double* egptr=Egamma;
      double* tgptr=Tgamma;
      char* ogptr=Ogamma;
      for(int k=0;k<Ngamma;k++){
	//
	for (int l=0;l<3;l++)r[l]=*rgptr++;
	double rdotr = r[0]*r[0]+r[1]*r[1]+r[2]*r[2];
	//
	// nothing to do if r>R1
	//
	if(sqrt(rdotr)<RC.R1){
	  for (int l=0;l<3;l++)dr[l]=rpmt[l]-r[l];
	  double ddr = sqrt(dr[0]*dr[0]+dr[1]*dr[1]+dr[2]*dr[2]);
	  for (int l=0;l<3;l++)tr[l]=dr[l]/ddr;
	  double tdotr = r[0]*tr[0]+r[1]*tr[1]+r[2]*tr[2];
	  //
	  // solve for intersection with inner radius
	  //
	  double sGd;
	  double sSc;
	  //
	  // check for no intersections
	  //
	  double test = RC.R0*RC.R0+tdotr*tdotr-rdotr;
	  bool inter = (test>=0);
	  //
	  // A straight line from the interaction vertex to the pmt
	  // goes through the inner radius at least once.  In this case,
	  // the light will see Gd-scint. and unloaded scint.
	  //
	  if(inter){
	    double sint1 = -tdotr+sqrt(test);
	    double sint2 = -tdotr-sqrt(test);
	    if(sint1>0&&sint2<0){     // 1 intersection at soln 1
	      sGd = sint1;
	    }
	    else if(sint2>0&&sint1<0){     // 1 intersection at soln 2
	      sGd = sint2;
	    }
	    else if(sint2>0&&sint1>0){     // 2 intersections 
	      sGd = sint1-sint2;
	      if(sGd<0)sGd*=-1;
	    }
	    else{                          // no intersections towards PMT
	      sGd = 0;
	    }
	    sSc = ddr-sGd;
	  }
	  //
	  //  otherwise the light only goes through unloaded scintillator
	  //
	  else{
	    sGd = 0;
	    sSc = ddr;
	  }
	  //
	  //  increment PH-- make atten. relative to center
	  //
	  double dsGd = sGd-RC.R0;
	  double dsSc = sSc-RC.R2+RC.R0;
	  double atten = exp(-dsGd/RC.attenlGd-dsSc/RC.attenlSc);     
	  MeanAtten+=atten/Npmt/Ngamma;
	  //
	  // solid angle factor
	  //
	  double sfactor=0;
	  for (int l=0;l<3;l++){sfactor+=tr[l]*rpmt[l]/RC.R2;}
	  sfactor *= RC.R2*RC.R2/ddr/ddr/Npmt;
	  MeanSfactor+=sfactor/Ngamma;
	  //
	  // photostatistics
	  //
	  double phgen = *egptr++;
	  float xpe = phgen*atten*sfactor*RC.PMTcoverage*RC.PhotonsPerMeV;     
	  int npe;
	  int error;
	  rnpssn_(xpe,npe,error);
	  MeanNpe+=1.0*npe/Npmt/Ngamma;
	  
	  double phsmr = phgen*atten*sfactor*(npe/xpe);     
	  
	  double tgen = *tgptr++;
	  double tsmr = RC.refracGd*sGd/RC.c+RC.refracSc*sSc/RC.c + tgen; 
	  
	  if(*ogptr++ == 'P'){
	    PHpositron+=phsmr;
	    sumpos[0]+=phgen*npe/xpe;
	    sumpos[1]+=phgen;
	    sumpos[2]+=phgen*atten*sfactor;
	    sumpos[3]+=phgen*atten*atten*sfactor*sfactor;
	  }
	  else{
	    PHneutron+=phsmr;
	    sumneu[0]+=phgen*npe/xpe;
	    sumneu[1]+=phgen;
	    sumneu[2]+=phgen*atten*sfactor;
	    sumneu[3]+=phgen*atten*atten*sfactor*sfactor;
	  }
	  
	  PMT PMTtmp;
	  PMTtmp.npmt = pmtindex;
	  for(int l=0;l<3;l++)PMTtmp.xyz[l] = rpmt[l];
	  PMTtmp.PHgen = phgen;
	  PMTtmp.PHsmr = phsmr;
	  PMTtmp.PathGd = sGd;
	  PMTtmp.PathSc = sSc;
	  PMTtmp.Atten = atten;
	  PMTtmp.Sfactor = sfactor;
	  PMTtmp.TimeGen = tgen;
	  PMTtmp.TimeSmr = tsmr;
	  PMTtmp.Xpe = xpe;
	  PMTtmp.Npe = npe;
	  PMTtmp.Parent = *ogptr;
	  
	  if(PMTptr<PMTdata+PMTsize){
	    *PMTptr++=PMTtmp;
	    PMTlen++;
	  }
	  else{
	    //
	    // expand the data array first
	    //
	    PMTptr = ResizePMT();
	    *PMTptr++=PMTtmp;
	    PMTlen++;
	  }
	}
      }
      //
      //  Crude estimate of reconstructed PH
      //  Includes poisson npe smearing and fluctuation in atten length
      //
      float rn[2];
      int len=2;
      rnormx_(rn,len,ranlux_);
      if(sumpos[2]>0){
	double sigpos = 
	  (sumpos[3]*sumpos[1]/sumpos[2]/sumpos[2]-1);
	if(sigpos>0){sigpos=sqrt(sigpos);}else{sigpos=0;}
	PHposReco+=sumpos[0]*(1+sigpos*rn[0])/Npmt;
      }
      if(sumneu[2]>0){
	double signeu = 
	  (sumneu[3]*sumneu[1]/sumneu[2]/sumneu[2]-1);
	if(signeu>0){signeu=sqrt(signeu);}else{signeu=0;}
	PHneuReco+=sumneu[0]*(1+signeu*rn[1])/Npmt;
      }
      //
      // increment PMT index
      //
      pmtindex++;
    }
  }
}
void ReactorDetector::GenerateGammas(ReactorEvent& Event){
  //
  int maxgamma = 25;
  int ngamma=0;
  double* egamma = new double[maxgamma];
  double* tgamma = new double[maxgamma];
  double* rgamma = new double[3*maxgamma];
  char* ogamma = new char[maxgamma];
  //
  double* egptr=egamma;
  double* rgptr=rgamma;
  double* tgptr=tgamma;
  char* ogptr=ogamma;
  //
  // get positron, neutron vertices
  //
  double rpos[3];
  double rneu[3];
  for(int i=0;i<3;i++)rpos[i] = *(Event.GetPositronVertexXYZ()+i);
  for(int i=0;i<3;i++)rneu[i] = *(Event.GetNeutronVertexXYZ()+i);
  double tpos = Event.GetPositronTime();
  double tneu = Event.GetNeutronTime();
  double kepos = Event.GetPositronKE();
  //
  // positron gammas first
  // the first is a "fake" gamma with E=positron KE generated at positron
  // vertex.
  //
  for(int i=0;i<3;i++)*rgptr++=rpos[i];
  *tgptr++ = tpos;
  *egptr++ = kepos;
  *ogptr++ = 'P';
  ngamma++;
  //
  // Now the two annihilation gammas
  //
  int len=4;
  float rv[4];
  ranlux_(rv,len);
  double cosg = -1+2*rv[0];
  double phig = 2*RC.pi*rv[1];
  double t[3];
  t[0] = cos(phig)*sqrt(1-cosg*cosg);
  t[1] = sin(phig)*sqrt(1-cosg*cosg);
  t[2] = cosg;
  double range;
  for (int i=0;i<2;i++){
    range = GammaComptonRange(RC.Melectron);
    range*=log(1/rv[i+2]);
    if(i==0){
      for(int j=0;j<3;j++)*rgptr++ = rpos[j]+range*t[j];
    }
    if(i==1){
      for(int j=0;j<3;j++)*rgptr++ = rpos[j]-range*t[j];
    }
    *egptr++=RC.Melectron;
    *tgptr++=tpos+range/RC.c*RC.refracSc;; 
    *ogptr++ = 'P';
    ngamma++;
  }
  //
  // Generate neutron gammas
  //
  //
  // pick H or Gd, then which Gd peak
  //
  double rrneu = sqrt(rneu[0]*rneu[0]+rneu[1]*rneu[1]+rneu[2]*rneu[2]);
  double emax=RC.Hpeak;
  if(rrneu<RC.R0){//in Gd region
    len=2;
    ranlux_(rv,len);
    if(rv[0]<=RC.GdCaptureFraction){
      //
      // model Gd spectrum by Gd155 + Gd157
      //
      if(rv[1]<=RC.Gd155frac){emax = RC.Gd155peak;}
      else{emax = RC.Gd157peak;}
    }
  }
  //
  // One photon from an H capture
  //
  if(emax==RC.Hpeak){
    len=3;
    ranlux_(rv,len);
    range = GammaComptonRange(RC.Hpeak);
    range*=log(1/rv[0]);
    cosg = -1+2*rv[1];
    phig = 2*RC.pi*rv[2];
    t[0] = cos(phig)*sqrt(1-cosg*cosg);
    t[1] = sin(phig)*sqrt(1-cosg*cosg);
    t[2] = cosg;
    for(int j=0;j<3;j++)*rgptr++ = rneu[j]+range*t[j];
    *egptr++ = RC.Hpeak;
    *tgptr++ = tneu+range/RC.c*RC.refracSc;
    *ogptr++ = 'N';
    ngamma++;
  }
  //
  // otherwise model Gd photons
  //
  else{
    int ngd;
    double pgd[3*maxgamma];
    MyGdDecayModel(emax,ngd,pgd);
    float rv2[ngd];
    ranlux_(rv2,ngd);
    double* qgd=pgd;
    double check[4]={0,0,0,0};
    for(int i=0;i<ngd;i++){
      double p[3];
      for(int j=0;j<3;j++)p[j]=*qgd++;
      double e = sqrt(p[0]*p[0]+p[1]*p[1]+p[2]*p[2]);
      range = GammaComptonRange(e);
      range*=log(1/rv[i]);
      for(int j=0;j<3;j++)*rgptr++ = rneu[j]+range*p[j]/e;
      //
      for(int j=0;j<3;j++)check[j]+=p[j];
      check[3]+=e;
      //
      *egptr++ = e;
      *tgptr++ = tneu+range/RC.c*RC.refracGd;
      *ogptr++ = 'N';
      ngamma++;
    }
  }
  //
  // Transfer temp data to storage
  //
  delete[] Rgamma;
  delete[] Egamma;
  delete[] Tgamma;
  delete[] Ogamma;
  
  Ngamma = ngamma;
  Rgamma = new double[3*ngamma];
  Egamma = new double[ngamma];
  Tgamma = new double[ngamma];
  Ogamma = new char[ngamma];

  rgptr=rgamma+3*ngamma;
  double* ptr = Rgamma+3*ngamma;
  while(ptr>Rgamma)*--ptr=*--rgptr;

  egptr=egamma+ngamma;
  ptr = Egamma+ngamma;
  while(ptr>Egamma)*--ptr=*--egptr;

  tgptr=tgamma+ngamma;
  ptr = Tgamma+ngamma;
  while(ptr>Tgamma)*--ptr=*--tgptr;

  ogptr=ogamma+ngamma;
  char* qtr = Ogamma+ngamma;
  while(qtr>Ogamma)*--qtr=*--ogptr;

  delete[] rgamma;
  delete[] egamma;
  delete[] tgamma;
  delete[] ogamma;
    
}


ReactorDetector::PMT* ReactorDetector::ResizePMT(){

  int oldsize = PMTsize;
  int newsize = 2*oldsize;
  PMT* PMThold = new PMT[newsize];
  PMT* PMTold = PMTdata+oldsize;
  PMT* PMTnew = PMThold+oldsize;
  while(PMTnew>PMThold)*--PMTnew=*--PMTold;
  delete[] PMTdata;
  PMTdata = new PMT[newsize];
  PMTold = PMThold+newsize;
  PMTnew = PMTdata+newsize;
  while(PMTnew>PMTdata)*--PMTnew=*--PMTold;
  delete[] PMThold;
  PMTsize = newsize;
  return PMTdata+oldsize;
}
//
// Fetch number of gammas from positron
//
int ReactorDetector::GetNgamma(char parent){
  int gamma=0;
  for(char* optr=Ogamma;optr<Ogamma+Ngamma;optr++){
    if (*optr==parent)gamma++;
  }
  return gamma;
}
//
// Fetch array of positron gammas
//
void ReactorDetector::GetGamma(char parent,double* xyz){

  double* Rptr=Rgamma;
  double* rptr=xyz;
  //
  for(char* optr=Ogamma;optr<Ogamma+Ngamma;optr++){
    if (*optr==parent){
      for(int i=0;i<3;i++)*rptr++ = *Rptr++;
    }
  }
}  
void ReactorDetector::GetGamma(char parent,double* e,
				  double* t,double* r,
				  double* c,double* f){
  double* Eptr=Egamma;
  double* Rptr=Rgamma;
  double* Tptr=Tgamma;
  double* eptr=e;
  double* rptr=r;
  double* tptr=t;
  double* cptr=c;
  double* fptr=f;
  //
  for(char* optr=Ogamma;optr<Ogamma+Ngamma;optr++){
    if (*optr==parent){
      *e++=*Eptr;
      *t++=*Tptr;
      double x=*(Rptr+0);
      double y=*(Rptr+1);
      double z=*(Rptr+2);
      *r++=sqrt(x*x+y*y+z*z);
      *c++=z/(*r);
      *f++=atan2(y,x);
      if((*f)<0)*f+=2*RC.pi;
    }
    Eptr++;
    Tptr++;
    Rptr+=3;
  }
}  
#endif






